/*
 * Copyright 2010-2015 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jetbrains.kotlin.idea.stubindex;

import com.intellij.openapi.project.Project;
import com.intellij.psi.search.GlobalSearchScope;
import com.intellij.psi.stubs.StringStubIndexExtension;
import com.intellij.psi.stubs.StubIndexKey;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.kotlin.psi.JetClassOrObject;

import java.util.Collection;

public class JetTopLevelClassByPackageIndex extends StringStubIndexExtension<JetClassOrObject> {
    private static final StubIndexKey<String, JetClassOrObject> KEY = KotlinIndexUtil.createIndexKey(JetTopLevelClassByPackageIndex.class);

    private static final JetTopLevelClassByPackageIndex ourInstance = new JetTopLevelClassByPackageIndex();

    public static JetTopLevelClassByPackageIndex getInstance() {
        return ourInstance;
    }

    private JetTopLevelClassByPackageIndex() {}

    @NotNull
    @Override
    public StubIndexKey<String, JetClassOrObject> getKey() {
        return KEY;
    }

    @Override
    public Collection<JetClassOrObject> get(String fqName, Project project, @NotNull GlobalSearchScope scope) {
        return super.get(fqName, project, JetSourceFilterScope.kotlinSourcesAndLibraries(scope, project));
    }
}
